// Copyright (2013) Jann Horn <jann@thejh.net>
// This code is licensed under the AGPLv3.

#include "common.h"

#include <unistd.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

extern char *rcon_host, *rcon_port, *rcon_pass, *map;

void senderr(char *errmsg, bool myfault) {
  if (myfault) {
    puts("Status: 500 CGI error");
  } else {
    puts("Status: 400 CGI error");
  }
  puts("Content-Type: text/plain;charset=utf8");
  puts("X-Frame-Options: DENY");
  puts("");
  puts("An error occured while processing your request:");
  puts(errmsg);
  exit(0);
}

struct inifile *get_config(void) {
  static struct inifile *res = NULL;
  if (!res) res = slurp_inifile(CONFIG_PATH);
  if (!res) {
    // error!
    senderr("can't fetch config", true);
  }
  return res;
}

char *getcval(char *key) {
  char *r = inifile_lookup(get_config(), key);
  if (!r) {
    // error!
    senderr("config value missing", true);
  }
  return r;
}

void rcon_send(char *data) {
  // get config
  char *host = getcval("rcon_host");
  char *port = getcval("rcon_port");
  char *pass = getcval("rcon_pass");
  
  // construct packet
  size_t pass_len = strlen(pass);
  size_t data_len = strlen(data);
  size_t packet_len = 4+4+1+pass_len+1+data_len +1;
  char buf[packet_len];
  char *p = buf;
  memcpy(p, "\xff\xff\xff\xffrcon ", 9); p += 9;
  memcpy(p, pass, pass_len); p += pass_len;
  *(p++) = ' ';
  memcpy(p, data, data_len); p += data_len;
  *(p++) = '\0';
  
  // connect
  int fd = socket(AF_INET, SOCK_DGRAM, 0);
  if (fd == -1) senderr("can't create UDP socket", true);
  struct sockaddr_in addr;
  bzero(&addr, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_port = htons(atoi(port));
  if (!inet_aton(host, &addr.sin_addr)) senderr("bad ipv4 addr in config", true);
  if (connect(fd, (struct sockaddr *)&addr, sizeof(addr)))
    senderr("can't connect UDP socket", true);
  
  // send
  if (write(fd, buf, packet_len) != packet_len)
    senderr("incomplete packet sent or write error", true);
  
  close(fd);
}
